package footfall.generators;

/**
  Generates the sound grains used to simulate footfalls (contact, knocks, thumps etc). The sound grains are usually 25-75ms in duration, depending on the user-selected frames per second.
  @author Chirag Mehta (chirag@chir.ag)
*/

public class SoundGrainModel
{
  // Default bitrate: 44.1khz
  private int bitRate = 44100;

  /**
    Provides functions for generating sound grains
  */
  public SoundGrainModel()
  {
    this(44100);
  }

  /**
    Provides functions for generating sound grains at specific bitrate
    @param bitRate Specific bitRate for sound grain generation: 11.025kHz - 44.1kHz
  */
  public SoundGrainModel(int bitRate)
  {
    if(bitRate > 11025)
      this.bitRate = bitRate;
    else
      this.bitRate = 11025;
  }

  /**
    Returns the bit rate of the sound grains. Should be the same as the bit rate of the GrainPlayer and MultiPlayer classes.
  */
  public int getBitRate()
  {
    return this.bitRate;
  }

  /**
    Generate sine wave that starts at given volume and then fades out
    @param bytes Length of the grain
    @param freq Frequency of the sine wave
    @param startAmp Starting amplitude of the wave
  */
  public float[] makeFadingSineWave(int bytes, int freq, float startAmp)
  {
    float[] buf = new float[bytes];
    for(int i=0; i < bytes; i++)
      buf[i] = (float)(Math.sin(i/(float)this.bitRate*freq*Math.PI) * startAmp * (bytes-i)/(float)bytes);
    return buf;
  }

  /**
    Generate white noise at given frequency and amplitude
    @param bytes Length of the grain
    @param freq Frequency of the white noise
    @param maxAmp Maximum amplitude of the white noise
  */
  public float[] makeWhiteNoise(int bytes, int freq, float maxAmp)
  {
    float[] buf = new float[bytes];
    float noiseVal = (float)((Math.random()*2d - 1d) * maxAmp);
    if(freq > this.bitRate) freq=this.bitRate;

    int intervalH = (int)this.bitRate/freq;
    float intervalV = 0;
    int nextI = 0;

    for(int i = 0; i < bytes; i++)
    {
      if(i == nextI)
      {
        buf[i] = noiseVal;
        noiseVal = (float)((Math.random()*2d - 1d) * maxAmp);
        intervalV = (float)((noiseVal - buf[i])/(float)intervalH);
        nextI += intervalH;
      }
      else
        buf[i] = (float)(buf[i-1] + intervalV);
    }
    return buf;
  }
}