package footfall.players;

/**
  Input sound grains and uses Java Sound to render audio.
  @author Chirag Mehta (chirag@chir.ag)
*/

public class GrainPlayer
{
  // At 44.1kHz
  private int bitRate = 11025 * 4;

  // Internally volume is stored as (0 = minimum, 127 = maximum) but for vertical scrollbars it is returned via getVolume() as 127-volume.
  private int volume = 127;

  // Current buffer to store grains
  private float[] currentBuf;

  // Sound player with 32 channels for smooth non-blocking playback
  MultiPlayer multiplayer;

  /**
    Input sound grains and uses Java Sound to render audio.
  */
  public GrainPlayer()
  {
    this(44100);
  }

  /**
    Input sound grains and uses Java Sound to render audio.
    @param bitRate Playback bit rate (11.025kHz -44.1kHz)
  */
  public GrainPlayer(int bitRate)
  {
    if(bitRate > 11025)
      this.bitRate = bitRate;
    else
      this.bitRate = 11025;

    multiplayer = new MultiPlayer(bitRate, bitRate/5, 32);
  }

  /**
    Add grain to current buffer (additive). Merges with the contents currently in buffer.
    @param b Grain buffer to add
  */
  public void addGrain(float[] b)
  {
    for(int i = 0; i < b.length && i < currentBuf.length; i++)
      currentBuf[i] += b[i];
  }

  /**
    Playback bit rate
  */
  public int getBitRate()
  {
    return this.bitRate;
  }

  /**
    Volume for use in vertical scrollbars (127 = minimum, 0 = maximum)
  */
  public int getVolume()
  {
    return 127-this.volume;
  }

  /**
    Volume Label for use in scrollbars (0% - 100%)
  */
  public String getVolumeLabel()
  {
    return String.valueOf((int)(this.volume*100/127f));
  }

  /**
    Clear current buffer on every frame and load new buffer of specific size
    @param bufSize Size New buffer size
  */
  public void initGrains(int bufSize)
  {
    this.currentBuf = new float[bufSize];
  }

  /**
    Play the grains in the current buffer and return the buffer for display
  */
  public float[] playGrains()
  {
    for(int i = 0; i < this.currentBuf.length; i++)
    {
      currentBuf[i] = (int)(currentBuf[i] * this.volume);
      if(currentBuf[i] > 127) currentBuf[i] = 127;
      if(currentBuf[i] < -127) currentBuf[i] = -127;
      multiplayer.queue(currentBuf[i]);
      currentBuf[i] /= 127f;
    }
    multiplayer.play();
    return this.currentBuf;
  }

  /**
    Set the volume/amplitude for the grain player
    @param volume Volume (127 = minimum, 0 = maximum)
  */
  public void setVolume(int volume)
  {
    this.volume = 127-volume;
  }

  /**
    Start the lines for audio output
  */
  public void start()
  {
    multiplayer.start();
  }

  /**
    Stop the lines for audio output
  */
  public void stop()
  {
    multiplayer.stop();
  }
}