package footfall.players;

import java.io.*;
import java.util.Vector;
import javax.sound.sampled.*;

/**
  Sound player with multiple channels for smooth non-blocking playback
  @author Chirag Mehta (chirag@chir.ag)
*/

public class MultiPlayer
{
  // Audio lines
  private AudioFormat audioformat;
  private SourceDataLine.Info info;
  private SourceDataLine line[];

  // Current audio queue/buffer
  private Vector serialqueue = new Vector();

  // Audio setup
  private int bitRate = 11025 * 4;
  private int bufSize = 11025;
  private int channels = 32;
  private int currentChannel = 0;

  /**
    Sound player with multiple channels for smooth non-blocking playback
  */
  public MultiPlayer()
  {
    this(44100, 11025, 32);
  }

  /**
    Sound player with multiple channels for smooth non-blocking playback
    @param bitRate Playback bit rate
    @param bufSize Buffer size for each write to audio channel
    @param channels Number of simultaneous channels to open
  */
  public MultiPlayer(int bitRate, int bufSize, int channels)
  {
    // Check whether bitRate is within bounds: 11.025kHz - 44.1kHz
    if(bitRate > 11025)
      this.bitRate = bitRate;
    else
      this.bitRate = 11025;

    this.bufSize = bufSize;
    this.channels = channels;

    // Allocate audio lines
    line = new SourceDataLine[channels];

    // Open each line/channel
    try
    {
      audioformat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, this.bitRate, 8, 1, 1, this.bitRate, true);
      for(int i = 0; i < channels; i++)
      {
        info = new DataLine.Info(SourceDataLine.class, audioformat, this.bitRate);
        line[i] = (SourceDataLine) AudioSystem.getLine(info);
        line[i].open(audioformat);
      }
    }
    catch (LineUnavailableException e) { System.out.println(currentChannel); e.printStackTrace();}
  }

  /**
    Clear buffer queue
  */
  public void clear()
  {
    serialqueue.clear();
  }

  /**
    If buffered queue length is large as buffer size minimum, write to next audio channel
  */
  public void play()
  {
    int qs = serialqueue.size();

    if(qs < bufSize)
      return;

    byte[] b = new byte[qs];
    for(int i = 0; i < qs; i++)
      b[i] = (byte)(((Byte)(serialqueue.get(i))).byteValue());

    line[currentChannel].write(b, 0, qs);
    currentChannel = (currentChannel + 1) % this.channels;
    clear();
  }

  /**
    Enqueue byte value
    @param b Amplitude
  */
  public void queue(byte b)
  {
    serialqueue.add(new Byte(b));
  }

  /**
    Enqueue double value
    @param d Amplitude
  */
  public void queue(double d)
  {
    serialqueue.add(new Byte((byte)d));
  }

  /**
    Enqueue float value
    @param f Amplitude
  */
  public void queue(float f)
  {
    serialqueue.add(new Byte((byte)f));
  }

  /**
    Enqueue integer value
    @param i Amplitude
  */
  public void queue(int i)
  {
    serialqueue.add(new Byte((byte)i));
  }

  /**
    Start each channel, ready for audio playback
  */
  public void start()
  {
    for(int i = 0; i < this.channels; i++)
      line[i].start();
  }

  /**
    Flush all open channels and stop playback
  */
  public synchronized void stop()
  {
    for(int i = 0; i < this.channels; i++)
    {
      line[i].flush();
      line[i].stop();
    }
  }
}