package footfall.synths;

import footfall.displays.*;
import footfall.generators.*;
import footfall.players.*;
import footfall.streams.*;

/**
  Abstract class for synthesizing footfall sounds based pressure data. Two ways to synthesize granular sounds: (1) by whole frames or (2) by each individual foot pressure cell.
  @author Chirag Mehta (chirag@chir.ag)
*/

public abstract class FootFallSynth implements Runnable
{
  // Primary foot fall stream to input footfall movements from
  FootFallStream footfallstream;

  // Oscilloscope display
  DisplaySound displaysound;

  // Used to generate white noise and fading sine waves
  SoundGrainModel soundgrainmodel;

  // Used for playing sound grains
  GrainPlayer grainplayer;

  // Thread info
  private volatile Thread thread;
  private volatile boolean threadDead = true;

  // Used for rendering oscilloscope
  private float[] historyBuf;

  // On-screen settings for various sound grain parameters
  float crunchiness = 0.0f;
  float compression = 0.0f;
  float grainsize   = 0.0f;
  float carpet      = 0.0f;
  float woodenness  = 0.0f;
  float creakiness  = 0.0f;

  /**
    Abstract class for synthesizing footfall sounds based pressure data. Sets the appropriate bit rate and initializes the grain generator and player by calling {@link #FootFallSynth(FootFallStream)}.
  */
  public FootFallSynth()
  {
    this(null);
  }

  /**
    Abstract class for synthesizing footfall sounds based pressure data with given footfall stream as input. Sets the appropriate bit rate and initializes the grain generator and player.
  */
  public FootFallSynth(FootFallStream footfallstream)
  {
    int bitRate = 44100;

    this.footfallstream = footfallstream;
    soundgrainmodel = new SoundGrainModel(bitRate);
    grainplayer = new GrainPlayer(bitRate);
  }

  /**
    Returns the value for the specified string parameter
    @param sParam One of the following: crunchiness, compression, grainsize, carpet, woodenness, creakiness
  */
  public float getParam(String sParam)
  {
    if(sParam.equals("crunchiness"))
      return this.crunchiness;
    else if(sParam.equals("compression"))
      return this.compression;
    else if(sParam.equals("grainsize"))
      return this.grainsize;
    else if(sParam.equals("carpet"))
      return this.carpet;
    else if(sParam.equals("woodenness"))
      return this.woodenness;
    else if(sParam.equals("creakiness"))
      return this.creakiness;
    return 0;
  }

  /**
    Get the grain player for purposes like setting volume, retrieving bit rate etc.
  */
  public GrainPlayer getPlayer()
  {
    return this.grainplayer;
  }

  /**
    Set the oscilloscope to specified display
    @param displaysound Oscilloscope Display panel
  */
  public void setDisplay(DisplaySound displaysound)
  {
    this.displaysound = displaysound;
  }

  /**
    Set the input footfall stream
    @param footfallstream New footfall stream
  */
  public void setFootFallStream(FootFallStream footfallstream)
  {
    this.footfallstream = footfallstream;
  }

  /**
    Thread's run method to {@link #update} the current frame non-stop
  */
  public void run()
  {
    threadDead = false;
    Thread me = Thread.currentThread();

    while (thread == me)
    {
      update();
      try { thread.sleep(0, 1); }
      catch(InterruptedException e) { }
    }

    thread = null;
    threadDead = true;
  }

  /**
    Set the value for the specified string parameter
    @param sParam One of the following: crunchiness, compression, grainsize, carpet, woodenness, creakiness
    @param nVal Float value to set
  */
  public void setParam(String sParam, float nVal)
  {
    if(sParam.equals("crunchiness"))
      this.crunchiness = nVal;
    else if(sParam.equals("compression"))
      this.compression = nVal;
    else if(sParam.equals("grainsize"))
      this.grainsize = nVal;
    else if(sParam.equals("carpet"))
      this.carpet = nVal;
    else if(sParam.equals("woodenness"))
      this.woodenness = nVal;
    else if(sParam.equals("creakiness"))
      this.creakiness = nVal;
  }

  /**
    Start footfall sound synthesis
  */
  public void start()
  {
    grainplayer.start();

    thread = new Thread(this, "footfallsynth");
    thread.setPriority(Thread.MAX_PRIORITY);
    thread.start();
  }

  /**
    Stop footfall sound synthesis
  */
  public void stop()
  {
    grainplayer.stop();
    thread = null;
    while(!threadDead);
  }

  /**
    Abstract method to retrieve the current frame and generate the appropriate sound via frame-based or cell-based granular synthesis
  */
  public abstract void synthesizeFootFallSound();

  /**
    Called by thread.run(). Initialize the grains. Generate/synthesize the sound. Play the grains. Display the oscilloscope if valid.
  */
  public void update()
  {
    this.grainplayer.initGrains(this.grainplayer.getBitRate()/footfallstream.getFramesPerSecond());
    synthesizeFootFallSound();
    this.historyBuf = this.grainplayer.playGrains();

    if(this.historyBuf != null && this.displaysound != null)
      displaysound.updateDisplay(historyBuf);
  }
}